# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import argparse
import getpass
import os
import time
import tomllib
import subprocess
import sys
from pathlib import Path

from sshare.logger         import Logger
from sshare.plugin         import Plugin
from sshare.plugin         import PluginManager
from sshare.version        import version


def main():
    # TODO: Add --config flag
    config_directory = Path(os.environ.get("XDG_CONFIG_DIR", f"{os.environ["HOME"]}/.config")) / "sshare"
    with open(config_directory / "config.toml", mode="rb") as file:
        config = tomllib.load(file)

    INTERNAL_PLUGIN_LOCATION = "sshare.plugins.default"
    # Load command line early and set it as the active logger
    # so that it can be used to report errors while loading and
    # configuring other loggers
    command_line = Plugin.internal(INTERNAL_PLUGIN_LOCATION, "command_line", config.get("plugins", dict()))
    logger = Logger(command_line=command_line)
    arg_parser = argparse.ArgumentParser(
        prog = "sshare",
        description = "Upload files to a server via ssh",
    )
    arg_parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s version {version}",
    )
    plugins = PluginManager(
        [ "logger", "source", "name", "upload", "result" ],
        logger,
        config.get("plugins", dict()),
        arg_parser,
    )
    plugins.add_from(
        Plugin.internal(INTERNAL_PLUGIN_LOCATION),
        "file",
        "current_time",
        "append_type",
        "ssh",
        "log_result",
    )
    sys.dont_write_bytecode = True
    plugins.add_from(
        Plugin.external,
        *[
            path for
            path in
            (config_directory / "plugins").iterdir()
            if path.is_file() and path.suffix == ".py"
        ]
    )
    sys.dont_write_bytecode = False
    plugins.activate("logger")
    logger.add(*plugins.logger.active)
    plugins.activate()

    sources = []
    for plugin in plugins.source.active:
        sources.append(plugin.get_source())
    if len(sources) == 0:
        logger.error("Error: No sources provided. Must activate at least one source plugin.")

    for index, source in enumerate(sources):
        name = ""
        for plugin in plugins.name.active:
            name = plugin.get_name(name, source)
        sources[index] = name, source

    for name, source in sources:
        for plugin in plugins.upload.active:
            plugin.upload(name, source)

    for name, _ in sources:
        for plugin in plugins.result.active:
            plugin.result(name)

    sys.exit(0)
